import { combineLatest, merge, of } from 'rxjs';
import { ignoreElements, map, switchMap, tap, withLatestFrom, } from 'rxjs/operators';
import { getTNode, notifyAllParentsIfNeeded, notifyInjectingParentIfNeeded, } from './utils';
import { onStrategy } from '../utils/onStrategy';
import { strategyHandling } from '../utils/strategy-handling';
import { getTemplateHandler } from './list-view-handler';
export function createListTemplateManager(config) {
    const { templateSettings, renderSettings, trackBy, iterableDiffers } = config;
    const { defaultStrategyName, strategies, cdRef: injectingViewCdRef, patchZone, parent, eRef, } = renderSettings;
    const strategyHandling$ = strategyHandling(defaultStrategyName, strategies);
    const differ = iterableDiffers.find([]).create(trackBy);
    //               type,  payload
    const tNode = parent
        ? getTNode(injectingViewCdRef, eRef.nativeElement)
        : false;
    /* TODO (regarding createView): this is currently not in use. for the list-manager this would mean to provide
     functions for not only create. developers than should have to provide create, move, remove,... the whole thing.
     i don't know if this is the right decision for a first RC */
    const listViewHandler = getTemplateHandler(Object.assign(Object.assign({}, templateSettings), { initialTemplateRef: templateSettings.templateRef, patchZone }));
    const viewContainerRef = templateSettings.viewContainerRef;
    let notifyParent = false;
    let changesArr;
    let partiallyFinished = false;
    return {
        nextStrategy(nextConfig) {
            strategyHandling$.next(nextConfig);
        },
        render(values$) {
            return values$.pipe(render());
        },
    };
    function render() {
        let count = 0;
        return (o$) => o$.pipe(
        // map iterable to latest diff
        map((iterable) => {
            if (partiallyFinished) {
                const currentIterable = [];
                for (let i = 0, ilen = viewContainerRef.length; i < ilen; i++) {
                    const viewRef = viewContainerRef.get(i);
                    currentIterable[i] = viewRef.context.$implicit;
                }
                differ.diff(currentIterable);
            }
            return {
                changes: differ.diff(iterable),
                items: iterable != null && Array.isArray(iterable) ? iterable : [],
            };
        }), withLatestFrom(strategyHandling$.strategy$), 
        // Cancel old renders
        switchMap(([{ changes, items }, strategy]) => {
            if (!changes) {
                return [];
            }
            const listChanges = listViewHandler.getListChanges(changes, items);
            changesArr = listChanges[0];
            const insertedOrRemoved = listChanges[1];
            const applyChanges$ = getObservablesFromChangesArray(changesArr, strategy, count);
            partiallyFinished = true;
            // @TODO we need to know if we need to notifyParent on move aswell
            notifyParent = insertedOrRemoved && parent;
            count = items.length;
            return merge(combineLatest(
            // emit after all changes are rendered
            applyChanges$.length > 0 ? [...applyChanges$] : [of(items)]).pipe(tap(() => (partiallyFinished = false)), 
            // somehow this makes the strategySelect work
            notifyAllParentsIfNeeded(tNode, injectingViewCdRef, strategy, () => notifyParent)), 
            // emit injectingParent if needed
            notifyInjectingParentIfNeeded(injectingViewCdRef, strategy, insertedOrRemoved).pipe(ignoreElements()));
        }));
    }
    function getObservablesFromChangesArray(changes, strategy, count) {
        return changes.length > 0
            ? changes.map((change) => {
                return onStrategy(change, strategy, (_change) => {
                    const type = _change[0];
                    const payload = _change[1];
                    switch (type) {
                        case 0 /* insert */:
                            listViewHandler.insertView(payload[0], payload[1], count);
                            break;
                        case 2 /* move */:
                            listViewHandler.moveView(payload[2], payload[0], payload[1], count);
                            break;
                        case 1 /* remove */:
                            listViewHandler.removeView(payload);
                            break;
                        case 3 /* update */:
                            listViewHandler.updateView(payload[0], payload[1], count);
                            break;
                        case 4 /* context */:
                            listViewHandler.updateUnchangedContext(payload[1], count);
                            break;
                    }
                }, {});
            })
            : [];
    }
}
//# sourceMappingURL=list-template-manager.js.map