import { zoneGlobalDisableConfigurationsKeys, zoneGlobalEventsConfigurationsKeys, zoneGlobalSettingsConfigurationsKeys, zoneRuntimeConfigurationsKeys, zoneTestDisableConfigurationsKeys, zoneTestSettingsConfigurationsKeys, } from './model/configurations.types';
const zoneDisable = '__Zone_disable_';
const zoneSymbol = '__zone_symbol__';
/**
 * factory function to create a `ZoneConfig` object.
 *
 * @Example
 * import { globalEvents,xhrEvent, zoneConfig} from '@rx-angular/cdk/zone-flags';
 *
 * const zoneConfig = createZoneFlagsConfigurator();
 *
 * zoneConfig.global.disable.requestAnimationFrame();
 * zoneConfig.global.disable.timers();
 * zoneConfig.events.disable.UNPATCHED_EVENTS([...globalEvents, ...xhrEvent]);
 *
 */
function createZoneFlagsConfigurator() {
    const cfg = window;
    const configProps = [
        ...[
            ...zoneGlobalDisableConfigurationsKeys,
            ...zoneTestDisableConfigurationsKeys,
        ].map((prop) => zoneDisable + prop),
        ...[
            ...zoneGlobalSettingsConfigurationsKeys,
            ...zoneTestSettingsConfigurationsKeys,
            ...zoneGlobalEventsConfigurationsKeys,
            ...zoneRuntimeConfigurationsKeys,
        ].map((prop) => zoneSymbol + prop),
    ];
    // append as global method for easy debugging
    cfg.__rax_zone_config__log = () => {
        configProps.forEach((flag) => {
            // tslint:disable-next-line:no-unused-expression
            cfg[flag] && console.log(flag, cfg[flag]);
        });
    };
    return {
        global: {
            disable: zoneGlobalDisableConfigurationsKeys
                .map((prop) => ({
                [prop]: (isDisabled = true) => (cfg[zoneDisable + prop] = isDisabled),
            }))
                .concat(zoneGlobalSettingsConfigurationsKeys.map((prop) => ({
                [prop]: (isDisabled = true) => (cfg[zoneSymbol + prop] = isDisabled),
            })))
                .reduce((map, item) => (Object.assign(Object.assign({}, map), item)), {}),
        },
        test: {
            disable: zoneTestDisableConfigurationsKeys
                .map((prop) => ({
                [prop]: (isDisabled = true) => (cfg[zoneDisable + prop] = isDisabled),
            }))
                .concat(zoneTestSettingsConfigurationsKeys.map((prop) => ({
                [prop]: (isDisabled = true) => (cfg[zoneSymbol + prop] = isDisabled),
            })))
                .reduce((map, item) => (Object.assign(Object.assign({}, map), item)), {}),
        },
        events: {
            disable: zoneGlobalEventsConfigurationsKeys
                .map((prop) => ({
                [prop]: (eventNames) => (cfg[zoneSymbol + prop] = [
                    ...(Array.isArray(cfg[zoneSymbol + prop])
                        ? cfg[zoneSymbol + prop]
                        : []),
                    ...eventNames,
                ]),
            }))
                .reduce((map, item) => (Object.assign(Object.assign({}, map), item)), {}),
        },
        runtime: {
            disable: zoneRuntimeConfigurationsKeys.reduce((map, prop) => (Object.assign(Object.assign({}, map), { [prop]: (isDisabled = true) => (cfg[zoneSymbol + prop] = isDisabled) })), {}),
        },
    };
}
export const zoneConfig = createZoneFlagsConfigurator();
//# sourceMappingURL=zone-cfg.js.map